@@ -1,29 +1,29 @@
 use crate::client::Client;
+use crate::uploader::cos::Cos;
 use crate::uploader::kodo::Kodo;
 use crate::uploader::upos::Upos;
 use crate::uploader::Uploader;
-use crate::Video;
-use anyhow::{bail, Context, Result};
-use async_std::fs::File;
-use futures::TryStreamExt;
+use crate::{Video, VideoFile, VideoStream};
+use anyhow::{Context, Result};
+use futures::{Stream, TryStreamExt};
+use reqwest::Body;
 use serde::de::DeserializeOwned;
 use serde::{Deserialize, Serialize};
 use serde_json::json;
-use std::path::{Path, PathBuf};
+use std::ffi::OsStr;
+
 use std::time::Instant;
 
 pub struct Parcel<'a> {
     line: &'a Line,
-    filepath: PathBuf,
+    video_file: VideoFile,
     params: serde_json::Value,
-    pub total_size: u64,
 }
 
 impl<'a> Parcel<'a> {
-    async fn new(line: &'a Line, filepath: &Path) -> Result<Parcel<'a>> {
-        let file = File::open(&filepath).await?;
-        let total_size = file.metadata().await?.len();
-        let file_name = filepath.file_name().ok_or("No filename").unwrap().to_str();
+    fn new(line: &'a Line, video_file: VideoFile) -> Parcel<'a> {
+        let total_size = video_file.total_size;
+        let file_name = video_file.file_name.clone();
         let profile = if let Uploader::Upos = line.os {
             "ugcupos/bup"
         } else {
@@ -33,22 +33,21 @@ impl<'a> Parcel<'a> {
             "r": line.os,
             "profile": profile,
             "ssl": 0,
-            "version": "2.8.12",
-            "build": 2081200,
+            "version": "2.11.0",
+            "build": 2110000,
             "name": file_name,
             "size": total_size,
         });
         println!("pre_upload: {}", params);
-        Ok(Self {
+        Self {
             line,
-            filepath: filepath.into(),
             params,
-            total_size,
-        })
+            video_file,
+        }
     }
 
     pub async fn pre_upload<T: DeserializeOwned>(&self, login: &Client) -> Result<T> {
-        Ok(login
+        login
             .client
             .get(format!(
                 "https://member.bilibili.com/preupload?{}",
@@ -59,39 +58,46 @@ impl<'a> Parcel<'a> {
             .await?
             .json()
             .await
-            .with_context(|| "Failed to pre_upload from".to_string())?)
+            .with_context(|| "Failed to pre_upload from".to_string())
     }
 
-    pub async fn upload(
-        &self,
-        client: &Client,
-        limit: usize,
-        mut process: impl FnMut(usize) -> bool,
-    ) -> Result<Video> {
-        let file = File::open(&self.filepath).await?;
-        // let total_size = file.metadata().await?.len();
-        // let file_name = filepath.file_name().ok_or("No filename")?.to_str();
-        match self.line.os {
+    pub async fn upload<F, S, B>(&self, client: &Client, limit: usize, progress: F) -> Result<Video>
+    where
+        F: FnOnce(VideoStream) -> S,
+        S: Stream<Item = Result<(B, usize)>>,
+        B: Into<Body> + Clone,
+    {
+        let mut video = match self.line.os {
             Uploader::Upos => {
-                let bucket = self.pre_upload(client).await?;
+                let bucket: crate::uploader::upos::Bucket = self.pre_upload(client).await?;
+                let chunk_size = bucket.chunk_size;
                 let upos = Upos::from(bucket).await?;
                 let mut parts = Vec::new();
-                let stream = upos.upload_stream(file, limit).await?;
+                let stream = upos
+                    .upload_stream(
+                        progress(self.video_file.get_stream(chunk_size)?),
+                        self.video_file.total_size,
+                        limit,
+                    )
+                    .await?;
                 tokio::pin!(stream);
-                while let Some((part, size)) = stream.try_next().await? {
+                while let Some((part, _size)) = stream.try_next().await? {
                     parts.push(part);
-                    if !process(size) {
-                        bail!("移除视频");
-                    }
                 }
-                upos.get_ret_video_info(&parts, &self.filepath).await
+                upos.get_ret_video_info(&parts, &self.video_file.filepath)
+                    .await?
             }
             Uploader::Kodo => {
                 let bucket = self.pre_upload(client).await?;
+                let chunk_size = 4194304;
                 Kodo::from(bucket)
                     .await?
-                    .upload_stream(file, &self.filepath, limit, process)
-                    .await
+                    .upload_stream(
+                        progress(self.video_file.get_stream(chunk_size)?),
+                        self.video_file.total_size,
+                        limit,
+                    )
+                    .await?
             }
             Uploader::Bos => {
                 panic!()
@@ -100,9 +106,30 @@ impl<'a> Parcel<'a> {
                 panic!()
             }
             Uploader::Cos => {
-                panic!()
+                let bucket = self.pre_upload(client).await?;
+                let cos_client = Cos::form_post(bucket).await?;
+                let chunk_size = 10485760;
+                let enable_internal = self.line.probe_url == "internal";
+                let parts = cos_client
+                    .upload_stream(
+                        progress(self.video_file.get_stream(chunk_size)?),
+                        self.video_file.total_size,
+                        limit,
+                        enable_internal,
+                    )
+                    .await?;
+                cos_client.merge_files(parts).await?
             }
-        }
+        };
+        if video.title == None {
+            video.title = self
+                .video_file
+                .filepath
+                .file_stem()
+                .and_then(OsStr::to_str)
+                .map(|s| s.to_string())
+        };
+        Ok(video)
     }
 }
 
@@ -159,8 +186,8 @@ pub struct Line {
 }
 
 impl Line {
-    pub async fn to_uploader(&self, filepath: &Path) -> Result<Parcel<'_>> {
-        Parcel::new(self, filepath).await
+    pub fn to_uploader(&self, filepath: VideoFile) -> Parcel<'_> {
+        Parcel::new(self, filepath)
     }
 }
 
@@ -169,7 +196,7 @@ impl Default for Line {
         Line {
             os: Uploader::Upos,
             probe_url: "//upos-sz-upcdnbda2.bilivideo.com/OK".to_string(),
-            query: "upcdn=bda2&probe_version=20200810".to_string(),
+            query: "upcdn=bda2&probe_version=20211012".to_string(),
             cost: u128::MAX,
         }
     }
@@ -178,7 +205,7 @@ impl Default for Line {
 pub fn kodo() -> Line {
     Line {
         os: Uploader::Kodo,
-        query: "bucket=bvcupcdnkodobm&probe_version=20200810".into(),
+        query: "bucket=bvcupcdnkodobm&probe_version=20211012".into(),
         probe_url: "//up-na0.qbox.me/crossdomain.xml".into(),
         cost: 0,
     }
@@ -187,7 +214,7 @@ pub fn kodo() -> Line {
 pub fn bda2() -> Line {
     Line {
         os: Uploader::Upos,
-        query: "upcdn=bda2&probe_version=20200810".into(),
+        query: "upcdn=bda2&probe_version=20211012".into(),
         probe_url: "//upos-sz-upcdnbda2.bilivideo.com/OK".into(),
         cost: 0,
     }
@@ -196,7 +223,7 @@ pub fn bda2() -> Line {
 pub fn ws() -> Line {
     Line {
         os: Uploader::Upos,
-        query: "upcdn=ws&probe_version=20200810".into(),
+        query: "upcdn=ws&probe_version=20211012".into(),
         probe_url: "//upos-sz-upcdnws.bilivideo.com/OK".into(),
         cost: 0,
     }
@@ -205,8 +232,26 @@ pub fn ws() -> Line {
 pub fn qn() -> Line {
     Line {
         os: Uploader::Upos,
-        query: "upcdn=qn&probe_version=20200810".into(),
+        query: "upcdn=qn&probe_version=20211012".into(),
         probe_url: "//upos-sz-upcdnqn.bilivideo.com/OK".into(),
         cost: 0,
     }
 }
+
+pub fn cos() -> Line {
+    Line {
+        os: Uploader::Cos,
+        query: "&probe_version=20211012&r=cos&profile=ugcupos%2Fbupfetch&ssl=0&version=2.10.4.0&build=2100400&webVersion=2.0.0".into(),
+        probe_url: "".into(),
+        cost: 0,
+    }
+}
+
+pub fn cos_internal() -> Line {
+    Line {
+        os: Uploader::Cos,
+        query: "".into(),
+        probe_url: "internal".into(),
+        cost: 0,
+    }
+}
